# !/usr/bin/python
import urllib2
import bs4
import sys
import os
import time


def is_stale(player_name):
    file_age = time.time() - os.path.getmtime(player_name)

    if file_age > 86400:
        return True
    else:
        return False


def check_cache(player_name):
    file = player_name

    if os.path.isfile(file):
        return True
    else:
        return False


def download_file(player_name):
    url = 'http://alt.org/nethack/player-all-xlog.php?player=%s' % (os.path.basename(player_name).split('.')[0])

    response = urllib2.urlopen(url)

    html = response.read()

    with open(player_name, 'wb') as f:
        f.write(html)

    return html


def get_player_file(player_list):
    '''
    This function will save the associated player_name html file
    # TODO accept a list of player names and download in threads.
    # TODO cache files for a length of time so we aren't always hitting the server.
    '''

    for p in player_list.split(','):

        file = '/tmp/' + p + '.html'

        if check_cache(file):

            if not is_stale(file):
                print "File is still fresh. Using local cached file for %s." % (p)
            else:
                print "file is stale, downloading xlog for %s." % (p)
                download_file(file)
        else:
            print "No cache found for %s, downloading." % (p)
            download_file(file)

    return file


def process_html(player_file):
    '''
    Processes the html file(s) of the users
    :param player_file:
    :return: lists of things
    '''
    player_name, dates, scores, roles, ascension_games = [], [], [], [], []
    player_games = {}
    with open(player_file, 'r') as file:
        soup = bs4._soup(file)
    try:
        games = soup.findAll('pre')[0].string.split('\n')
    except IndexError:
        print "No games found for user %s. Maybe check spelling?" % os.path.basename(player_file).split('.')[0]
        os.remove(player_file)
        sys.exit(99)

    for i in range(0, len(games) - 1):
        keys = [t.split('=')[0] for t in games[i].split(':')]
        values = [t.split('=')[1] for t in games[i].split(':')]
        all_games = dict(zip(keys, values))

        dates.append(str(all_games['deathdate']))
        scores.append(int(all_games['points']))
        roles.append(str(all_games['role']))

        if str(all_games['death']) == 'ascended':
            ascension_games.append(all_games)

    return dates, scores, roles, ascension_games



import sys
import utils
from prettytable import PrettyTable

table = PrettyTable(['Player', 'Total Games', 'Ascensions', 'Pct', 'Avg. Turns', 'Fastest',
                     'Slowest', 'Total Turns', 'Total Points', 'Avg. Points', 'Time Spent'])


def process_ascensions(player_dict):
    for p in player_dict:
        if len(player_dict[p]['ascension_games']) == 0:
            print "No ascensions for %s. Total games: %s" % (p, len(player_dict[p]['dates']))
            sys.exit(99)

        roles = [str(g['role']) for g in player_dict[p]['ascension_games']]
        race = [str(g['race']) for g in player_dict[p]['ascension_games']]
        gender0 = [str(g['gender0']) for g in player_dict[p]['ascension_games']]
        gender = [str(g['gender']) for g in player_dict[p]['ascension_games']]
        align0 = [str(g['align0']) for g in player_dict[p]['ascension_games']]
        align = [str(g['align']) for g in player_dict[p]['ascension_games']]
        points = [int(g['points']) for g in player_dict[p]['ascension_games']]
        turns = [int(g['turns']) if g['turns'] else 0 for g in player_dict[p]['ascension_games']]
        dates = [str(g['deathdate']) for g in player_dict[p]['ascension_games']]
        maxlvl = [int(g['maxlvl']) for g in player_dict[p]['ascension_games']]
        realtime = [int(g['realtime']) if g['realtime'] else 0 for g in player_dict[p]['ascension_games']]
        total_games = len(player_dict[p]['dates'])
        dhm = utils.seconds_to_days_hours_mins(sum(realtime))

        if len(player_dict) == 1:
            print ''
            print '{:^30}'.format('[ Ascensions ]')
            print ''
            print '{:30} {:,}'.format('Total Games', total_games)
            print '{:30} {:,} ({:.2%})'.format('Ascensions', len(points), float(len(points)) / float(total_games))
            print '{:30} {:,}'.format('Avg. turns/ascension', (sum(turns)) / len(turns))
            print '{:30} {:,}'.format('Fastest Ascension', min(turns))
            print '{:30} {:,}'.format('Slowest Ascension', max(turns))
            print '{:30} {:,}'.format('Total Games', total_games)

            print '{:30} {:,}'.format('Average turns/ascension', (sum(turns)) / len(turns))
            print '{:30} {:,}'.format('Fastest Ascension', min(turns))
            print '{:30} {:,}'.format('Slowest Ascension', max(turns))
            print ''
            print '{:30} {:,}'.format('Total Ascension Turns', sum(turns))
            print ''
            print '{:30} {:,}'.format('Total Points', sum(points))
            print '{:30} {:,}'.format('Avg. Points/Ascension', (sum(points)) / (len(points)))
            print '{:30} {} days {} hours {} minutes'.format('Time Spent Ascending', dhm[0], dhm[1], dhm[2])
            sys.exit(0)

        table.add_row([p, len(player_dict[p]['dates']),
                       int(len(points)),
                       str(round((float(len(points)) / float(len(player_dict[p]['dates']))), 5) * 100) + ' %',
                       sum(turns) / len(turns), min(turns), max(turns), sum(turns),
                       sum(points), (sum(points)) / (len(points)),
                       str(dhm[0]) + 'd ' + str(dhm[1]) + 'h ' + str(dhm[2]) + 'm '])

        table.align = 'r'

    table.sortby = 'Avg. Turns'
    print table



#!/usr/bin/env python
import nao
import format
import argparse
import os


def main():
    # TODO make saving the file optional
    # TODO make a pretty table
    """ Fetches a users NAO data"""
    parser = argparse.ArgumentParser()
    parser.add_argument('--list', '-l', help='NAO user list')
    cmd_args = parser.parse_args()

    if not cmd_args.list:
        parser.print_help()
    elif cmd_args.list:
        player_list = [p.strip() for p in cmd_args.list.split(',')]
    print "Fetching games for user(s) %s" % cmd_args.list.split(',')

    players = [nao.get_player_file(p) for p in player_list]


    player_dict = {}
    for p in players:
        dates, scores, roles, ascension_games = nao.process_html(p)
        player_data = dict()
        player_data['dates'] = dates
        player_data['scores'] = scores
        player_data['roles'] = roles
        player_data['ascension_games'] = ascension_games
        player_dict[os.path.basename(p).split('.')[0]] = player_data

    format.process_ascensions(player_dict)


if __name__ == '__main__':
    main()



import datetime


def seconds_to_days_hours_mins(seconds):
    days = seconds / 86401
    hours = (seconds % 86400) / 3600
    minutes = ((seconds % 86400) % 3600) / 60

    return days, hours, minutes


lookup_z = [round(1 / float(n), 5) for n in range(1, 200)]


def timestamp_to_date(unix_timestamp):
    '''
    Returns the date in format 'YYYY-mm-dd'
    :param unix_timestamp:
    :return:
    '''
    value = datetime.datetime.fromtimestamp(unix_timestamp)
    return value.strftime('%Y-%m-%d')


def timestamp_to_datetime(unix_timestamp):
    '''
    Returns the date and time in format 'YYYY-mm-dd H:M:S'
    :param unix_timestamp:
    :return:
    '''
    value = datetime.datetime.fromtimestamp(unix_timestamp)
    return value.strftime('%Y-%m-%d %H:%M:%S')


def timestamp_to_time(unix_timestamp):
    '''
    Returns just the time component 'H:M:S'
    :param unix_timestamp:
    :return:
    '''
    value = datetime.datetime.fromtimestamp(unix_timestamp)
    return value.strftime('%H:%M:%S')


def days_between_dates(first, second):
    '''
    returns an int of the number of days between two dates
    the format of the dates input should be: YYYY-MM-DD
    :param first:
    :param second:
    :return: int
    '''

    first_date = datetime.datetime.strptime(first, '%Y-%M-%d')
    second_date = datetime.datetime.strptime(second, '%Y-%M-%d')

    if second_date > first_date:
        return (second_date - first_date).days
    else:
        return (first_date - second_date).days



import os
from setuptools import setup

# Utility function to read the README file.  
# Used for the long_description.  It's nice, because now 1) we have a top level
# README file and 2) it's easier to type in the README file than to put a raw
# string in below ...
def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()


requires = ['beautifulsoup4>=4.3.2', 'prettytable==0.7.2']

setup(
    name="ascension_kit",
    version='0.0.11',
    author="Greg Moselle",
    author_email="zomgreg+ascensionkit@gmail.com",
    description="A utility to report ascension information from NAO.",
    license="BSD",
    scripts=['bin/ascension-kit'],
    keywords="python nethack utility",
    url="http://packages.python.org/ascension_kit",
    packages=['ascension_kit'],
    long_description=read('README.rst'),
    include_package_data=True,
    install_requires=requires,
    zip_safe=False,
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Topic :: Utilities",
        "License :: OSI Approved :: BSD License",
    ],
)
